/*      $Archive: /General Utilities/CPUID/cpuname.cpp $
	$Author : $
	$Date   : $
	$Header : $
	$History: cpuname.cpp $
 * 
 * *****************  Version 7  *****************
 * User: Hnguyen      Date: 12/04/96   Time: 2:52a
 * Updated in $/General Utilities/CPUID
 * 
 * *****************  Version 6  *****************
 * User: Tanb         Date: 7/18/96    Time: 4:56p
 * Updated in $/General Utilities/CPUID
 * Minor changed in display
 */


#include "DEFINES.H"
#include <iostream.h>
#include <iomanip.h>
#include <stdlib.h>
#include <conio.h>

//cpu_signature identifies the specific CPU by providing information
//regarding the type, instruction family, model, stepping revision and
//the feature flags indicates the existence or presence of specific
//features.

void cpuid :: std_cpu_signature (void)
{
  int signature = 0;             //CPU signature variable
  int stepping_id = 0;           //CPU stepping id variable
  int model = 0;                 //CPU model variable
  int inst_family = 0;           //CPU instruction fammily variable
  unsigned int reg_ax = 0 ;      //AX register
  unsigned long reg_eax,reg_edx, test_reg;    //EAX, EDX, and test register variables
  unsigned long print_eax,print_ebx,print_ecx,print_edx;   //Display variable
  int maxbit = 18;                //Control loop variable
  int bits ;

  asm {
		 mov EAX,1        //EAX = 1 or function 1
		 db 0x0F, 0xA2    //CPUID opcode

  }
  //display the value of the registers
  print_eax = _EAX;
  print_ebx = _EBX;
  print_ecx = _ECX;
  print_edx = _EDX;

  reg_edx = _EDX;       //store the standard feature flags
  reg_ax = _AX;        
  asm mov BX, reg_ax
  asm  and BL,0x0F     //mask the right most 4 bits
  stepping_id = _BL;   //to get the cpu stepping id

  asm  mov BX, reg_ax
  asm  and BL,0xF0    //mask the left most 4 bits
  asm  ror BL,4       //to get the cpu model
  model = _BL;

  asm mov BX, reg_ax  //get the cpu instruction family
  asm and BH, 0x0F
  inst_family = _BH;

  asm and EAX,0xFFFFF000  //get the bits[31-12] 
  asm ror EAX,12
  reg_eax = _EAX;

  asm mov BX, reg_ax   //get the cpu signature 
  asm and BX,0x0FF0
  signature = _BX;

  clrscr();
  cout.setf(ios::uppercase);
  cout << "Function 1 (EAX = 1)" << endl;
  cout << "====================";
  cout << "\n\n";
  cout << "EAX == "<<setw(8)<<hex<<print_eax;
  cout << "  EBX == "<<setw(8)<<hex<<print_ebx;
  cout << "  ECX == "<<setw(8)<<hex<<print_ecx;
  cout << "  EDX == "<<setw(8)<<hex<<print_edx;
  cout.unsetf(ios::uppercase);
  cout << "\n\n";
  cout << "     EAX[3:0]   == " << setw(1) << hex << stepping_id << endl;
  cout << "     EAX[7:4]   == " << setw(1) << hex << model << endl;
  cout << "     EAX[11:8]  == " << setw(1) << hex << inst_family << endl;
  cout << "     EAX[31:12] == " << setw(5) << hex << reg_eax << endl;


  cout << "\n\n";
  cout << "     Processor Signature : ";

  if (signature == 0x0500)
	 cout << "AMD-K5 (Model 0) " << endl;
  else if (signature == 0x0510)
	 cout << "AMD-K5 (Model 1) " << endl;
  else if (signature == 0x0520)
	 cout << "AMD-K5 (Model 2) " << endl;
  else if (signature == 0x0530)
	 cout << "AMD-K5 (Model 3) " << endl;
  else if (signature == 0x0560)
	 cout << " AMD-K6 " << endl;
  else if (signature == 0x0400)
	 cout << "Am486 and Am5X86 " << endl;
  cout << "\n\n             Press any key for more."<<"\n\n";
  getch();

  clrscr();
  cout << "\n";
  cout << "     Feature Flags : " << "\n\n";
  cout << "          EDX == ";
  cout.setf(ios::uppercase);
  cout << setw(8) << hex << reg_edx << "\n\n";
  cout.unsetf(ios::uppercase);

  //get the standard feature flags
  for ( bits = 0; bits < maxbit; bits++){
	 switch (bits) {
		case 0 : test_reg = reg_edx;
			 if((test_reg & 0x00000001)== 0x00000001){    //test bit 0
					  cout.width(13);
					  cout.setf(ios::left);
					  cout << "EDX[0]  = 1b ";
					  cout.unsetf(ios::left);
					  cout << " (bit 0==1 indicates FPU present)" << endl;
					}
					else {
					  cout.width(13);
					  cout.setf(ios::left);
					  cout << "EDX[0]  = 0b ";
					  cout.unsetf(ios::left);
					  cout << " (bit 0==1 indicates FPU present)" << endl;
					}
					test_reg = reg_edx;
					break;
		case 1 : if ((test_reg & 0x00000002 )==0x00000002){  //test bit 1
					  cout.width(13);
					  cout.setf(ios::left);
					  cout << "EDX[1]  = 1b ";
					  cout.unsetf(ios::left);
					  cout << " (bit 1==1 indicates Virtual Mode Extensions)"
							 << endl;
					}
					else {
					  cout.width(13);
					  cout.setf(ios::left);
					  cout << "EDX[1]  = 0b ";
					  cout.unsetf(ios::left);
					  cout << " (bit 1==1 indicates Virtual Mode Extensions)"
							 << endl;
					}
					test_reg = reg_edx;
					break;
		case 2 : if ((test_reg & 0x00000004 )==0x00000004){   //test bit 2
					  cout.width(13);
					  cout.setf(ios::left);
					  cout << "EDX[2]  = 1b ";
					  cout.unsetf(ios::left);
					  cout << " (bit 2==1 indicates Debugging Extensions)" << endl;
					}
					else {
					  cout.width(13);
					  cout.setf(ios::left);
					  cout << "EDX[2]  = 0b ";
					  cout.unsetf(ios::left);
					  cout << " (bit 2==1 indicates Debugging Extensions )" << endl;
					}
					test_reg = reg_edx;
					break;

		case 3 : if ((test_reg & 0x00000008 )==0x00000008){   //test bit 3
					  cout.width(13);
					  cout.setf(ios::left);
					  cout << "EDX[3]  = 1b ";
					  cout.unsetf(ios::left);
					  cout << " (bit 3==1 indicates Page Size Extensions)" << endl;
					}
					else {
					  cout.width(13);
					  cout.setf(ios::left);
					  cout << "EDX[3]  = 0b ";
					  cout.unsetf(ios::left);
					  cout << " (bit 3==1 indicates Page Size Extensions)" << endl;
					}
					test_reg = reg_edx;
					break;
		case 4 : if ((test_reg & 0x00000010 )==0x00000010){    //test bit 4
					  cout.width(13);
					  cout.setf(ios::left);
					  cout << "EDX[4]  = 1b ";
					  cout.unsetf(ios::left);
					  cout << " (bit 4==1 indicates Time Stamp Counter)" << endl;
					}
					else {
					  cout.width(13);
					  cout.setf(ios::left);
					  cout << "EDX[4]  = 0b ";
					  cout.unsetf(ios::left);
					  cout << " (bit 4==1 indicates Time Stamp Counter )"<< endl;
					}
					test_reg = reg_edx;
					break;
		case 5 : if ((test_reg & 0x00000020 )==0x00000020){   //test bit 5
					  cout.width(13);
					  cout.setf(ios::left);
					  cout << "EDX[5]  = 1b ";
					  cout.unsetf(ios::left);
					  cout << " (bit 5==1 indicates K86 Model Specific Registers)"
							 << endl;
					}
					else {
					  cout.width(13);
					  cout.setf(ios::left);
					  cout << "EDX[5]  = 0b ";
					  cout.unsetf(ios::left);
					  cout << " (bit 5==1 indicates K86 Model Specific Registers)"
							 << endl;
					}
					test_reg = reg_edx;
					break;
		case 6 : if ((test_reg & 0x00000040 )==0x00000000){   //test bit 6
					  cout.width(12);
					  cout.setf(ios::left);
					  cout << "EDX[6]  = 0b";
					  cout.unsetf(ios::left);
					  cout << "  Reserved" << endl;
					}
					test_reg = reg_edx;
					break;
		case 7 : if ((test_reg & 0x00000080 )==0x00000080){  //test bit 7
					  cout.width(13);
					  cout.setf(ios::left);
					  cout << "EDX[7]  = 1b ";
					  cout.unsetf(ios::left);
					  cout << " (bit 7==1 indicates Support of Machine";
					  cout << " Check Exception)" << endl;
					}
					else {
					  cout.width(13);
					  cout.setf(ios::left);
					  cout << "EDX[7]  = 0b ";
					  cout.unsetf(ios::left);
					  cout << " (bit 7==1 indicates Support of Machine";
					  cout << " Check Exception)" << endl;
					}
					test_reg = reg_edx;
					break;
		case 8 : if ((test_reg & 0x00000100 )==0x00000100){    //test bit 8
					  cout.width(13);
					  cout.setf(ios::left);
					  cout << "EDX[8]  = 1b ";
					  cout.unsetf(ios::left);
					  cout << " (bit 8==1 indicates Support of CMPXCHG8B";
					  cout << " Extensions)" << endl;
					}
					else {
					  cout.width(13);
					  cout.setf(ios::left);
					  cout << "EDX[8]  = 0b ";
					  cout.unsetf(ios::left);
					  cout << " (bit 8==1 indicates Support of CMPXCHG8B";
					  cout << " Extensions)" << endl;
					}
					test_reg = reg_edx;
					break;
		case 9 : if ((test_reg & 0x00000200 )==0x00000200){ //test bit 9
						if (signature == 0x0500){
						  cout.width(13);
						  cout.setf(ios::left);
						  cout << "EDX[9]  = 1b ";
						  cout.unsetf(ios::left);
						  cout << " (bit 9==1 indicates Support of Global";
						  cout << " Paging Extension)"<< endl;
						}
						else {
						  cout.width(13);
						  cout.setf(ios::left);
						  cout << "EDX[9]  = 1b ";
						  cout.unsetf(ios::left);
						  cout << " (bit 9==1 indicates Support of APIC " << endl;
						}
					}
					else {
					  if (signature == 0x0500){
						 cout.width(13);
						 cout.setf(ios::left);
						 cout << "EDX[9]  = 0b ";
						 cout.unsetf(ios::left);
						 cout << " (bit 9==1 indicates Support of Global";
						 cout << " Paging Extensions)"<< endl;
						}
						else {
						  cout.width(13);
						  cout.setf(ios::left);
						  cout << "EDX[9]  = 0b ";
						  cout.unsetf(ios::left);
						  cout << " (bit 9==1 indicates Support of APIC " << endl;
						}
					}
					test_reg = reg_edx;
					break;
	  case 10 : if ((test_reg & 0x00000C00 )==0x00000000){ //test bit 10:11
					  cout.width(12);
					  cout.setf(ios::left);
					  cout << "EDX[10:11] = ";
					  cout.unsetf(ios::left);
					  cout << " Reserved"<< endl;
					}
					test_reg = reg_edx;
					break;
		case 11 : if ((test_reg & 0x00001000 )==0x00001000){ //test bit 12
					  cout.width(12);
					  cout.setf(ios::left);
					  cout << "EDX[12] = 1b";
					  cout.unsetf(ios::left);
					  cout << " (bit 12==1 indicates Memory Type Range Registers)"
							 << endl;
					}
					else {
					  cout.width(12);
					  cout.setf(ios::left);
					  cout << "EDX[12] = 0b ";
					  cout.unsetf(ios::left);
					  cout << " (bit 12==1 indicates Memory Type Range Registers)"
							 << endl;
					}
					test_reg = reg_edx;
					break;
	  case 12 : if ((test_reg & 0x00002000 )==0x00002000){ //test bit 13
					  cout.width(12);
					  cout.setf(ios::left);
					  cout << "EDX[13] = 1b ";
					  cout.unsetf(ios::left);
					  cout << " (bit 13==1 indicates Global Paging Extension)"
							 << endl;
					 }
					 else {
						 if (signature == 0x0500){
						 cout.width(12);
						 cout.setf(ios::left);
						 cout << "EDX[13] = 0b ";
						 cout.unsetf(ios::left);
						 cout << " Reserved" << endl;
						 }
						 else {
							cout.width(12);
							cout.setf(ios::left);
							cout << "EDX[13] = 0b ";
							cout.unsetf(ios::left);
							cout << " (bit 13==1 indicates Global Paging Extension)"
								  << endl;
						 }
					}
					test_reg = reg_edx;
					break;
	  case 13 : if ((test_reg & 0x00004000 )==0x00000000){ //test bit 14
						 cout.width(12);
						 cout.setf(ios::left);
						 cout << "EDX[14] = 0b ";
						 cout.unsetf(ios::left);
						 cout << " Reserved" << endl;
					 }
					test_reg = reg_edx;
					break;
	  case 14 : if ((test_reg & 0x00008000 )==0x00008000){ //test bit 15
					  cout.width(12);
					  cout.setf(ios::left);
					  cout << "EDX[15] = 1b ";
					  cout.unsetf(ios::left);
					  cout << " (bit 15==1 indicates Conditional Move Instruction)"
							 << endl;
					}
					else {
					  cout.width(12);
					  cout.setf(ios::left);
					  cout << "EDX[15] = 0b ";
					  cout.unsetf(ios::left);
					  cout << " (bit 15==1 indicates Conditional Move Instruction)"
							 << endl;
					}
					test_reg = reg_edx;
					break;
	  case 15 : if ((test_reg & 0x007F0000 )==0x00000000){ //test bit 16:22
					  cout.width(12);
					  cout.setf(ios::left);
					  cout << "EDX[16:22] = ";
					  cout.unsetf(ios::left);
					  cout << " Reserved" << endl;
					}
					test_reg = reg_edx;
					break;
	  case 16 : if ((test_reg & 0x00800000 )==0x00800000){ //test bit 23
					  cout.width(12);
					  cout.setf(ios::left);
					  cout << "EDX[23] = 1b ";
					  cout.unsetf(ios::left);
					  cout << " (bit 23==1 indicates Support of MultiMedia"
							 << " eXtensions)" << endl;
					}
					else {
					  cout.width(12);
					  cout.setf(ios::left);
					  cout << "EDX[23] = 0b ";
					  cout.unsetf(ios::left);
					  cout << " (bit 23==1 indicates Support of MultiMedia"
							 << " eXtensions)" << endl;
					}
					test_reg = reg_edx;
					break;
		case 17: if ((test_reg & 0xFF000000) == 0x00000000){  //test bit 24:31
					  cout.width(12);
					  cout.setf(ios::left);
					  cout << "EDX[24:31] = ";
					  cout.unsetf(ios::left);
					  cout << " Reserved" << endl;
					}

	 }
  }
  cout << "\n                   Press any key for more. " << endl;
  getch();
}
